@@ -32,7 +32,6 @@ class TextRLEnv(gym.Env):
         return reward
 
     def reset(self, input_text=None):
-        self.predicted = []
         if input_text is None:
             self.input_text = random.choice(self.observation_space)
         else:
@@ -41,17 +40,27 @@ class TextRLEnv(gym.Env):
 
     def _get_obs(self, predicted=[]):
         p_text = self.tokenizer.convert_tokens_to_string(predicted)
-        feature_dict = self.tokenizer([[self.input_text, p_text]], return_tensors='pt', add_special_tokens=False).to(
-            self.model.device)
-        prediction = self.model(**feature_dict, output_hidden_states=True)
-        outputs = prediction.decoder_hidden_states[-1].squeeze(0) if 'decoder_hidden_states' in prediction else prediction.hidden_states[-1].squeeze(0)
+        if hasattr(self.model.model, 'decoder'):
+            feature_dict = self.tokenizer([self.input_text], return_tensors='pt', add_special_tokens=False).to(
+                self.model.device)
+            predicted = [self.tokenizer.eos_token] + predicted
+            dec_input = torch.tensor([self.tokenizer.convert_tokens_to_ids(predicted)]).to(self.model.device)
+            feature_dict['decoder_input_ids'] = dec_input
+            prediction = self.model(**feature_dict, output_hidden_states=True)
+            outputs = prediction.decoder_hidden_states[-1].squeeze(0)
+        else:
+            feature_dict = self.tokenizer([[self.input_text, p_text]], return_tensors='pt',
+                                          add_special_tokens=False).to(self.model.device)
+            prediction = self.model(**feature_dict, output_hidden_states=True)
+            prediction.hidden_states[-1].squeeze(0)
         return outputs.data[-1]
 
     def _predict(self, vocab_id):
         predicted = self.predicted
         with torch.no_grad():
             pred_word = self.actions[vocab_id]
-            if pred_word == self.tokenizer.sep_token or len(pred_word) < 1:
+            if pred_word == self.tokenizer.sep_token or pred_word == self.tokenizer.eos_token \
+                    or len(pred_word) < 1 or len(self.predicted) > self.model.config.max_position_embeddings:
                 return predicted, True, self.tokenizer.convert_tokens_to_string(predicted)
             else:
                 predicted += [pred_word]
